package labss.lang.il.visitors;

import labss.lang.il.factories.ILEntityFactory;
import labss.lang.il.model.Addition;
import labss.lang.il.model.And;
import labss.lang.il.model.AssignmentStatement;
import labss.lang.il.model.BlockStatement;
import labss.lang.il.model.BooleanLiteral;
import labss.lang.il.model.Division;
import labss.lang.il.model.Equals;
import labss.lang.il.model.Expression;
import labss.lang.il.model.GreaterThan;
import labss.lang.il.model.IfElseStatement;
import labss.lang.il.model.IfStatement;
import labss.lang.il.model.IntLiteral;
import labss.lang.il.model.LessThan;
import labss.lang.il.model.Literal;
import labss.lang.il.model.Multiplication;
import labss.lang.il.model.Not;
import labss.lang.il.model.Or;
import labss.lang.il.model.Statement;
import labss.lang.il.model.Subtraction;
import labss.lang.il.model.Variable;
import labss.lang.il.model.WhileStatement;

import org.whole.lang.bindings.IBindingManager;
import org.whole.lang.model.IEntity;
import org.whole.lang.operations.InterpreterOperation;

/**
 * @author Riccardo Solmi
 */
public class ILInterpreterVisitor extends ILIdentityVisitor {
	private final IBindingManager opEnv, varEnv;

	public ILInterpreterVisitor(InterpreterOperation op) {
		wSetOperation(op);
		varEnv = op.getResultsEnvironment();
		opEnv = op.getOperationEnvironment();
	}

	protected final void setResult(Literal value) {
		opEnv.wDef("result", value);
	}
	protected final void setResult(boolean value) {
		setResult(ILEntityFactory.instance.createBooleanLiteral(value));
	}
	protected final void setResult(int value) {
		setResult(ILEntityFactory.instance.createIntLiteral(value));
	}
    protected final IEntity evaluate(Expression exp) {
    	exp.accept(this);
    	return opEnv.wGet("result");
    }
	protected final boolean booleanValue(Expression exp) {
		return evaluate(exp).wBooleanValue();
	}
	protected final int intValue(Expression exp) {
		return evaluate(exp).wIntValue();
	}
    protected final void execute(Statement stm) {
		stm.accept(this);
	}

	public void visit(BlockStatement stm) {
		for (int i=0; i<stm.wSize(); i++)
			((Statement) stm.wGet(i)).accept(this);
	}
	public void visit(WhileStatement stm) {
		while (booleanValue(stm.getCondition()))
			execute(stm.getTrueBody());
	}
	public void visit(IfStatement stm) {
		if (booleanValue(stm.getCondition()))
			execute(stm.getTrueBody());
	}
	public void visit(IfElseStatement stm) {
		if (booleanValue(stm.getCondition()))
			execute(stm.getTrueBody());
		else
			execute(stm.getFalseBody());
	}
	public void visit(AssignmentStatement stm) {
		varEnv.wDef(stm.getVariable().wStringValue(), evaluate(stm.getExpression()));
	}

	public void visit(BooleanLiteral entity) {
		setResult(entity.wBooleanValue());
	}

	public void visit(IntLiteral entity) {
		setResult(entity.wIntValue());
	}

	public void visit(Variable entity) {
		Literal result = (Literal) varEnv.wGet(entity.wStringValue());
		if (result == null)
			throw new IllegalStateException("Variabile non inizializzata: "+entity.wStringValue());
		setResult(result);
	}

	public void visit(Addition entity) {
		setResult(intValue(entity.getExp1()) + intValue(entity.getExp2()));
	}

	public void visit(Subtraction entity) {
		setResult(intValue(entity.getExp1()) - intValue(entity.getExp2()));
	}

	public void visit(Multiplication entity) {
		setResult(intValue(entity.getExp1()) * intValue(entity.getExp2()));
	}

	public void visit(Division entity) {
		setResult(intValue(entity.getExp1()) / intValue(entity.getExp2()));
	}

	public void visit(And entity) {
		setResult(booleanValue(entity.getExp1()) && booleanValue(entity.getExp2()));
	}

	public void visit(Or entity) {
		setResult(booleanValue(entity.getExp1()) || booleanValue(entity.getExp2()));
	}

	public void visit(Not entity) {
		setResult(!booleanValue(entity.getExp()));
	}

	public void visit(Equals entity) {
		setResult(intValue(entity.getExp1()) == intValue(entity.getExp2()));
	}

	public void visit(LessThan entity) {
		setResult(intValue(entity.getExp1()) < intValue(entity.getExp2()));
	}

	public void visit(GreaterThan entity) {
		setResult(intValue(entity.getExp1()) > intValue(entity.getExp2()));
	}

}
